

import java.io.*;
import java.util.*;

public class Personal implements Serializable {
    private List<Empleado> empleados;
    
    public Personal(Reader fpersonal) throws IOException {
        Scanner ent = null;
        empleados = new LinkedList<Empleado> ();
	try {
            ent = new Scanner(fpersonal);
            while (ent.hasNextLine()) {
                try {
                    Empleado e = new Empleado(ent.nextLine());
                    empleados.add(e);
                }
                catch (InputMismatchException e) {
                    continue;
                }
            }
        }
        finally {
            if (ent != null)
		ent.close();
        }
    }
    public Personal(DataInputStream fpersonal) throws IOException {
        empleados = new LinkedList<Empleado> ();
	try {
            while (true) {
                try {
                    Empleado e = new Empleado(fpersonal);
                    empleados.add(e);
                }
                catch (EOFException e) {
                    break;
                }
            }
        }
        finally {
            if (fpersonal != null)
		fpersonal.close();
        }
    }
    public Personal(ObjectInputStream ent) throws IOException, ClassNotFoundException {
        Personal p;
        p = (Personal) ent.readObject();
        this.empleados = p.empleados;
	ent.close();
    }
    public double nomina() {
        double total = 0;
        for (Empleado e: empleados)
            total += e.salario();

        return total;
    }

    public String toString() {
	String separador = System.getProperty("line.separator");
        String s = new String();
        for (Empleado e: empleados)
            s += e.toString() + separador;
	return s;
    }

    public void imprimir(PrintWriter sal) throws IOException {
        sal.println(this);
	sal.close();
    }
    public void imprimir(DataOutputStream sal) throws IOException {
        for (Empleado e: empleados)
            e.imprimir(sal);
	sal.close();
    }
    public void imprimir(ObjectOutputStream sal) throws IOException {
        sal.writeObject(this);
	sal.close();
    }
    public void imprimirNomina(String fsal) throws IOException {
	PrintWriter sal = new PrintWriter(fsal);
        sal.printf("%-10s|%-10s|%-10s|%-10s|%-10s|%-10s|%n", "Nombre",
                "Apellido 1", "Apellido 2", "Sal. base",
                "% ajuste", "Salario");
        sal.printf("-----------------------------------------------------------------%n");
        for (Empleado e: empleados) {
       	    double salario = e.salario();
            double base = e.getSueldoBase();
	    sal.printf("%-10s|%-10s|%-10s|%,10.2f|%,10.2f|%,10.2f|%n", e.getNombre(),
		e.getPrimerApellido(), e.getSegundoApellido(),
		base, ((salario-base)*100)/base, salario);

	}
        sal.printf("-----------------------------------------------------------------%n");
        sal.printf("%,65.2f%n", this.nomina());
        sal.printf("%n");
        sal.close();

    }
    // prueba de la clase personal
    static public void main(String [] args) throws IOException, ClassNotFoundException {
	if (args.length!=5) {
            System.err.println("Uso: Personal tipo_fent (1 texto, 2 data, 3 object) fent fsal_texto fsal_data_bin fsal_object_bin");
            return;
	}
        Personal empresa = null;
        int tipo = Integer.parseInt(args[0]);
        if (tipo==1)
            empresa = new Personal(new FileReader(args[1]));
        else if (tipo==2)
            empresa = new Personal(new DataInputStream(new FileInputStream(args[1])));
        else if (tipo==3)
            empresa = new Personal(new ObjectInputStream(new FileInputStream(args[1])));
        else {
            System.err.println("Uso: Personal tipo_fent (1 texto, 2 data, 3 object) fent fsal_texto fsal_data_bin fsal_object_bin");
            return;
	}
            
	empresa.imprimir(new PrintWriter(args[2]));
	empresa.imprimir(new DataOutputStream(new FileOutputStream(args[3])));
  	empresa.imprimir(new ObjectOutputStream(new FileOutputStream(args[4])));
    }
}
